
#include "fdtd.h"
#include "Timer.h"
#include <math.h>
#include <omp.h>
//#include <map>
//const double c0 = 300000000.0;
using namespace std;
FDTD::FDTD()
{
	qTime=0;
	 pauseSim=0;
	 m_TotalSteps = 0;
	// Create test data
	
	 m_time = 0;
	 m_outStep = 1;
	
	
};

	bool FDTD::isRunning() const
{
	return (pauseSim!=0);
}
void FDTD::togglePause()
{
	pauseSim=!pauseSim;
	
}
void FDTD::stepTime()
{


	m_time += 0.5*m_dt;

	
}

void FDTD::updateE()
{
	/* Calculate the Ez field */
	int k;
	

#pragma omp parallel for private(k) num_threads(7)
		for (k = 1; k < m_E.size(); k++)
		{
			m_E[k] = m_E[k] + (m_H[k] - m_H[k-1])*imp0;
		}
		//m_E[0] = exp(-(qTime - 30.) * (qTime - 30.) / 100.);
		//m_EzPointSensor.push_back(m_E[50]);
		/* use additive source at node 50 */
		m_E[50] += source_gauss(qTime,30,100);//  exp(-(qTime - 30.) * (qTime - 30.) / 100.);
}




void FDTD:: updateH()
{
/* Calculate the Hy field */
	int k;
	
	
#pragma omp parallel for private(k) num_threads(7)
				for ( k=0; k < m_H.size()-1; k++ )
				{ 
					m_H[k] = m_H[k]+(m_E[k+1] - m_E[k])/imp0;
				}
	
}

void FDTD::upDateSensor(vector<double>& pointsesor, double value)
{
	pointsesor.push_back(value);
}
void FDTD::displayPointSensorData(vector<double>& pointSensor)
{
	 int size = pointSensor.size();
	 if (size == 0) return;
	vector<double> timegrid = linspace(0, (double)size - 1, size);
	plot(timegrid, pointSensor);
}
void FDTD:: update()
{
	if (!isRunning()) return;
	static int frame = 1;
	Timer timer;
	timer.start();
				stepTime();
				updateH();
				stepTime();
				
				updateE();
				upDateSensor(m_EzPointSensor, m_E[50]);
				++qTime;
				timer.stop();	
				printf("Time for 1 step %lf microsec\n", timer.getElapsedTimeInMicroSec());
				if (qTime%m_outStep==0)
				{
					
					save_snapshot("Ez",frame,m_E);
					save_snapshot("Hy", frame, m_H);
					++frame;
				}
				

}

void FDTD::prepare4Run(int n, double dx)
{
	m_E.resize(n, 0.0);
	m_H.resize(n, 0.0);
	m_grid = linspace(0, (double)n-1.0, n);
	m_dx = dx;
	m_dt = m_dx /c0;
    #pragma omp parallel
    {
        int thread_num = omp_get_thread_num();
        int cpu_num = sched_getcpu();
        printf("Thread %3d is running on CPU %3d\n", thread_num, cpu_num);
    }


}
void FDTD::save_snapshot( const char *basename,int frame, field1d& field)
{
	char filename[3 * 256] = { "" };
	sprintf(filename, "%s_%04d.txt", basename, frame);
	FILE* snapshot = fopen(filename, "w");
	for (int mm = 0; mm < field.size(); mm++)
		fprintf(snapshot, "%g\n", field[mm]);
	fclose(snapshot);
}/*n*/
void tfst_Update_E(int pos, int qtime)
{

}
void tfsf_Update_H(int pos, int qtime)
{

}
void FDTD::DISPLAY()
{
	
	//plot
	layer("layer 1", 1);
	subplot(3, 1, 1);
	//axis(-2, 2, -2, 2, 0, 1);
	axis(0.0, m_grid.size(), -2E-0, 2E-0);
	title("Ez in space");
	plot(m_grid, m_E); set("r");//red
	subplot(3, 1, 2);
	title("Hz in space");
	plot(m_grid, m_H); set("b");//red

	
	layer("layer 2", 1);
	//mesh
	subplot(3, 1, 3);
	axis(0, m_EzPointSensor.size(), -1, 1);
	title("Ez at point x[50]");
	displayPointSensorData(m_EzPointSensor);

		return;

}
